clear;
close all;
%===========================
% Parameters
%===========================

%DGP
Xgrid = 0:0.001:1;
pargs = importdata('DGP.txt');
parg=pargs.data;
if parg(1)==1
    mu1 = parg(2:3); %strong bidder
    mu2=parg(4:5);  %weak bidder
    F1_grid = betacdf(Xgrid,mu1(1),mu1(2));
    F2_grid = betacdf(Xgrid,mu2(1),mu2(2));
else
    dis = importdata('DGP2.txt');
    Xgrid=dis(1,:);
    F1_grid=dis(2,:);
    F2_grid=dis(3,:);
    
end
Xinvgrid = -10:0.001:10;
Finvgrid = normcdf(Xinvgrid,0,1);
par_corr = importdata('corr.txt');
pbound = par_corr.data(4); %bound for the parameter space
ce=par_corr.data(1);
s= 1e-4;%step_size for polynormials
parg = importdata('pargs.txt');
Nsim = parg.data(5); %simulation draws
nbidder = parg.data(7);%number of bidders
con_opt = parg.data(6);% 1 if symmetry is imposed
nknots=parg.data(8);% number of knots for spline
nT=parg.data(10);% number of grids in numeric integration.

% hold on
% plot(Xgrid,F1_grid)
% plot(Xgrid,F2_grid,'red')
rng('shuffle')
% covariance matrix for copula
V = (ones(nbidder,nbidder)-eye(nbidder))*ce+ eye(nbidder);
V1 = V^(1/2);
N = parg.data(9); %sample size
iter=1;
v = cell(iter,1);
data = cell(iter,1);
for i =1 :iter;
    pre_v = randn(N,nbidder);
    pre_v1 = pre_v*V1;
    pre_v2 = normcdf(pre_v1,0,1);
    v{i,1}(:,1) = interp1(F1_grid,Xgrid,pre_v2(:,1));
    v{i,1}(:,2:nbidder) = interp1(F2_grid,Xgrid,pre_v2(:,2:nbidder));
    [a,b]=sort(v{i,1},2,'descend');
    data{i,1} = [a(:,2),b(:,1)==1];
end
% figure
% ecdf(data{1}(:,1));
% hold on
% plot(Xgrid,F1_grid,'red')
% plot(Xgrid,F2_grid,'red')


%===================================
%generate legendre polynomials
%===================================
knots=linspace(0,1,nknots);ngrid=1001;sorder=2;
[ Bsplines, DBsplines, vgrid] = Bspline_bases(knots,ngrid,sorder);
nvgrid = length(vgrid);
nstep = floor(nvgrid/10);
Bsplines_pen = Bsplines(1:nstep:end,:);
DBsplines_pen = DBsplines(1:nstep:end,:);
%=====================================
%inference options
%=====================================
T =linspace(-1,1,nT);
u = randn(Nsim,nbidder); %simulation draws
[~,n]=size(Bsplines);% number of polynomials
%===================
%pass variables
%===================
gvar.T=T;gvar.n=n;gvar.Bsplines=Bsplines;gvar.u=u;
gvar.vgrid=vgrid;gvar.Nsim=Nsim;
gvar.Bsplines_pen=Bsplines_pen;
gvar.nbidder=nbidder;gvar.pbound=pbound;
gvar.corr_lb = par_corr.data(2);
gvar.corr_ub = par_corr.data(3);
%===================================
%Obtain initial guesses
%===================================
testnull = importdata('null') ;
fx=testnull(1);

hnull =testnull(2);true = testnull(3);
ID = find(vgrid==fx);
pp = Bsplines(ID,:);
pp0 = Bsplines(1,:);
pp1 = Bsplines(end,:);
gvar.pp=pp;
options = optimset('algorithm','sqp','MaxFunEvals',50000);
Acon = -DBsplines; Bcon = zeros(length(vgrid),1);
Aeqs = [pp0;pp1;pp];beqs=[0;1;true];
Aeqw = [pp0;pp1];beqw=[0;1];

[deltas0,fval] = fmincon(@(deltas)mean((interp1(vgrid,Bsplines*deltas(:),Xgrid)-F1_grid).^2),zeros(1,n),Acon,Bcon,Aeqs,beqs,[],[],[],options); 
[deltaw0,fval] = fmincon(@(deltaw)mean((interp1(vgrid,Bsplines*deltaw(:),Xgrid)-F2_grid).^2),zeros(1,n),Acon,Bcon,Aeqw,beqw,[],[],[],options); 
% figure
% hold on
% plot(Xgrid,F1_grid,'red')
% plot(Xgrid,F2_grid,'magenta')
% plot(vgrid,Bsplines*deltas0(:))
% plot(vgrid,Bsplines*deltaw0(:))
% 
% 



%=================================
%Compute test statistic
%=================================
options=optimset('algorithm','sqp','MaxFunEvals',50000);
option=1;
delta0=[deltas0,deltaw0,0.3];
lb = [-ones(1,2*(n))*pbound,par_corr.data(2)];
ub = [ones(1,2*(n))*pbound,par_corr.data(3)];
beqs=[0;1;hnull];
Aeq = blkdiag(Aeqs,Aeqw);beq = [beqs;beqw];
Acon = [blkdiag(-DBsplines,-DBsplines);Bsplines,-Bsplines];bcon = zeros(ngrid*3,1);


[delta1,fval] = fmincon(@(delta)mean((interp1(vgrid,Bsplines*delta(1:n)',Xgrid)-F1_grid).^2)+mean((interp1(vgrid,Bsplines*delta(n+1:2*n)',Xgrid)-F2_grid).^2)...
    ,zeros(1,2*n),Acon,bcon,Aeq,beq,[],[],[],options);
Aeq = [Aeq,zeros(5,1)];
if con_opt==1
    Acon = [blkdiag(-DBsplines,-DBsplines);Bsplines,-Bsplines];bcon = zeros(ngrid*3,1);
else
    Acon = blkdiag(-DBsplines,-DBsplines);bcon = zeros(ngrid*2,1);
    Acon1 = [blkdiag(-DBsplines,-DBsplines);Bsplines,-Bsplines];bcon1 = zeros(ngrid*3,1);
    [nrows_Acon1,~] = size(Acon1);

    Acon1 = [Acon1,zeros(nrows_Acon1,1)];
    Acon_ext1 =[Acon1;Aeq;-Aeq];bcon_ext1 = [bcon1;beq;-beq];
end
[nrows_Acon,~] = size(Acon);

Acon = [Acon,zeros(nrows_Acon,1)];
Acon_ext =[Acon;Aeq;-Aeq];bcon_ext = [bcon;beq;-beq];


gvar.Acon=Acon;gvar.Aeq=Aeq;gvar.bcon=bcon;gvar.beq=beq;
B = parg.data(1);
fkappa1=importdata('kappa_func.txt');
fkappa=str2func(fkappa1{1});
kappa = fkappa(N);
gamma_grid = linspace(0,sqrt(kappa/N),parg.data(4));
q = [0.9,0.95];
Acon1 = [blkdiag(-DBsplines_pen,-DBsplines_pen);Bsplines_pen,-Bsplines_pen];
[ncon,~] = size(Acon1);bcon1 = zeros(ncon,1) ;
Acon1 = [Acon1,zeros(ncon,1)];
Aeq1=Aeq([1,2,4,5],:);beq1=[0;1;0;1];
delta1=[delta1,ce];

% [delta1,fval] = fmincon(@(delta)norm(delta3-delta),zeros(1,2*n+1),Acon,bcon,Aeq,beq,[],[],[],options); 


%================================================
%Bootstrap
%================================================
boot=cell(1,iter);
for i = 1:iter
    K=1;
    tvalue1=100;
    if con_opt==1
        [delta,tvalue]=fminsearchcon(@(delta)intmoment( data{i},delta,gvar),delta1,lb,ub,Acon_ext,bcon_ext+1e-12);
    else
        [deltaunc,tvalueunc]=fminsearchcon(@(delta)intmoment( data{i},delta,gvar),delta1,lb,ub,Acon_ext1,bcon_ext1+1e-12);
        [deltacon,tvaluecon]=fminsearchcon(@(delta)intmoment( data{i},delta,gvar),delta1,lb,ub,Acon_ext,bcon_ext+1e-12);
        if tvalueunc<tvaluecon
            delta=deltaunc;
            tvalue=tvalueunc;
        else
            delta=deltacon;
            tvalue=tvaluecon;
        end
    end
     
   %  [delta,tvalue]=fmincon(@(delta)intmoment( data{i},delta,gvar),delta0,Acon,bcon,Aeq,beq,lb,ub,[],options);
    while (abs(tvalue1-tvalue)>parg.data(3))&&(K<parg.data(2));
        delta1=delta;
        tvalue1=tvalue;
        [delta,tvalue]=fminsearchcon(@(delta)intmoment( data{i},delta,gvar),delta1,lb,ub,Acon_ext,bcon_ext+1e-12);
        K=K+1;
        
    end
    
    
    
    [y(i,:),boot{i}] = bootstrap(B,delta,data{i},q,gamma_grid,kappa,gvar );
end
fileID = fopen('output.txt','w');
fprintf(fileID,'%12.8f ',tvalue,y);
fclose(fileID);


